/*
	Programme permettant de :
		- Formuler un programme linaire en nombres entiers en utilisant un fichier .lp
		- Obtenir la solution de la relaxation 
		- Obtenir la solution entiere
		
	Utilisation de la librairie GLPK (GNU Linear Programming Kit)
*/

#include <stdio.h>
#include <glpk.h>

void creation_fichier_lp(char * name)
{
	/* Le fichier du plne */ 
	FILE * flot;

	/* Ouverture du fichier */
	flot = fopen( name, "w" ); 
	
	/* Ecriture de l'objectif */
	fprintf(flot, "\nMinimize\n\n");
	fprintf(flot, "obj: 0.3  x1 + 0.7 x2 + 0.2 x3 + 0.8 x4\n");
	

	/* Ecriture des contraintes */
	fprintf(flot, "\nSubject To\n\n");
	

	/* Contrainte 1 */
	fprintf( flot, "CNT1 : 0.9 x1 + 0.4 x2 >= 1\n");
	
	
	/* Contrainte 2 */
	fprintf( flot, "CNT2 : 0.8 x3 + 0.3 x4 >= 1\n");

	
	/* Contrainte 3 */
	fprintf( flot, "CNT3 : 0.8 x1 + 0.9 x4 >= 1\n");
	
	
	/* Le type des variables */	
	fprintf(flot, "\nBinary\n\n");

	fprintf(flot, "x1\n");
	fprintf(flot, "x2\n");
	fprintf(flot, "x3\n");
	fprintf(flot, "x4\n");
	
	fprintf(flot, "\nEnd\n");
	
	/* Fermeture du fichier */
	fclose(flot);
}


void afficher_relaxation(LPX * lp)
{
	int c;
	
	printf( "\nSolution de la relaxation\n" );
	
	printf( "Cout de la fonction = %lf\n", lpx_get_obj_val(lp) );
	
	printf( "Nombre de variables = %d\n", lpx_get_num_cols(lp) );

	for ( c = 1; c <= lpx_get_num_cols(lp); c++ )
		printf( "%s = %lf\n", lpx_get_col_name(lp,c), lpx_get_col_prim(lp,c) );
		
	printf( "\n" );
}

void afficher_solution_entiere(LPX * lp)
{
	int c;
	
	printf( "\nSolution entire\n" );
	
	printf( "Cout de la fonction = %lf\n", lpx_mip_obj_val(lp) );
	
	printf( "Nombre de variables = %d\n", lpx_get_num_cols(lp) );

	for ( c = 1; c <= lpx_get_num_cols(lp); c++ )
		printf( "%s = %lf\n", lpx_get_col_name(lp,c), lpx_mip_col_val(lp,c) );
		
	printf( "\n" );
}

int main()
{
	/* Le programme linaire */
	LPX * lp;

	/* Cration du fichier lp */
	creation_fichier_lp("test.lp");
	
	/* Chargement du problme */
	lp = lpx_read_cpxlp("test.lp");
	
	/* Lancement de la rsolution de la relaxation linaire */
	lpx_simplex(lp);
	
	/* Affichage de la relaxation */
	afficher_relaxation(lp);
	
	/* Lancement de la rsolution en nombres entiers */
	lpx_integer(lp);
	
	/* Affichage de la solution entire */
	afficher_solution_entiere(lp);

	/* Destruction du problme */	
	lpx_delete_prob(lp);
	
	return 1;
}
